/* $Id: mon_init_sh3.c,v 1.1.1.1 2001/12/29 00:25:32 amura Exp $ */
/*
 *  Copyright (c) 2001,2002    MURAMATSU Atsushi
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * A Simple Monitor Program
 *   - machine dependent initialize routines for HITACHI SH-3
 */

#include "macdep.h"
#include "monitor_in.h"
#include "monitor.h"

void
reset_handler_to_default(void)
{
    register int i;
    DISABLE();
    set_VBR(DEFAULT_VBR_ADDRESS);
    for (i=0; i<=TRAP_VECTOR_MAX; i++)
	set_trap_handler(i, dummy_handler);
    for (i=0; i<=INTR_VECTOR_MAX; i++)
	set_intr_handler(i, dummy_handler);
    ENABLE();
}

void
monitor_init(void)
{
    DISABLE();
    PC.CR.WORD = 0xAAAA;		/* all input */
    PE.CR.WORD = 0xAAAA;		/* all input */
    PF.CR.WORD = 0xAAAA;		/* all input */
    PG.CR.WORD = 0xAAAA;		/* all input */
    PH.CR.WORD = 0xAAAA;		/* all input */
    PJ.CR.WORD = 0x4000;		/* output is PJ bit7 */
    INTC.ICR0.WORD = 0x0000;		/* NMI is down edge */
    INT.ICR1.WORD = 0x4000;		/* default */
    INT.ICR2.WORD = 0x0000;		/* all IRQs are enabled by low level */
    reset_handler_to_default();
    serial_init();
}

/*** serial handlers ***/
void __recv_routine(void);
void __send_routine(void);
#ifdef HAVE_SLINE_INTR
void __line_check(void);
#endif

#ifdef __GNUC__
void __serial_bri(void) __attribute__((interrupt_handler));
void __serial_eri(void) __attribute__((interrupt_handler));
#else  /* for Hitachi SH-C Compiler */
#pragma interrupt(__serial_bri)
#pragma interrupt(__serial_eri)
#endif

void
__serial_bri(void)
{
    unsigned short dummy;
    dummy = SCIF.SCSSR.WORD;
    SCIF.SCSSR.WORD &= ~0x0010;
}

void
__serial_eri(void)
{
    unsigned short dummy;
    dummy = SCIF.SCSSR.WORD;
    SCIF.SCSSR.WORD &= ~0x0080;
}

void
__serial_init(void)
{
    volatile unsigned short dummy;
    volatile int count;

    set_IMASK(15);
    DISABLE();
    set_intr_handler(INTR_SCI2_ERI_VECTOR, __serial_eri);
    set_intr_handler(INTR_SCI2_RXI_VECTOR, __recv_routine);
    set_intr_handler(INTR_SCI2_BRI_VECTOR, __serial_bri);
    set_intr_handler(INTR_SCI2_TXI_VECTOR, __send_routine);

    INT.IPRE.BIT.LU = SERIAL_INTR_LEVEL & 0xF;
    SCIF.SCSMR.BYTE = 0;			/* set style 8N1 */
    SCIF.SCBRR = (unsigned char)(((1000000/DEFAULT_BIT_RATE) - 1) & 0xFF);
    						/* set bit rate */
    for (count=0; count<10000; count++)		/* wait 1 bit cycle */
	;
    SCIF.SCSCR.BYTE = 0x70;	/* start serial */
    dummy = SCIF.SCSSR.WORD;	/* dummy read */
    SCIF.SCSSR.WORD = 0x0060;	/* clear all status (except transmit intr) */
    SCIF.SCFCR.BYTE = 0x30;	/* set tx and rx trigger is 1, RTSCTS unuse */
    set_IMASK(0);
}

void
__serial_mode(long speed, int type)
{
    volatile int dummy;
    
    set_IMASK(15);
    SCIF.SCSCR.BYTE = 0x00;
    if (type == S_RTSCTS)		/* not RTSCTS to RTSCTS */
	SCIF.SCFCR.BIT.MCE = 0x31;
    else				/* RTSCTS to not RTSCTS */
	SCIF.SCFCR.BIT.MCE = 0x30;
    SCIF.SCBRR = (unsigned char)(((1000000/speed) - 1) & 0xFF);
    						/* set bit rate */
    for (dummy=0; dummy<10000; dummy++)		/* wait 1 bit cycle */
	;
    SCIF.SCSCR.BYTE = 0x70;	/* start serial */
    dummy = SCIF.SCSSR.WORD;	/* dummy read */
    SCIF.SCSSR.WORD = 0x0060;	/* clear all status (except transmit intr) */
    set_IMASK(0);
}
