/* $Id: mon_load.c,v 1.5 2002/01/13 16:36:41 amura Exp $ */
/*
 *  Copyright (c) 2001,2002    MURAMATSU Atsushi
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * A Simple Monitor Program
 *   - application program loding routine
 */

#include "macdep.h"
#include "monitor_in.h"
#include "monitor.h"

#include <ctype.h>

#define	TWIRL		"-\\|/"
#define TWIRL_NUM	4

static int
hex2toint(char *str)
{
    register int c, ret;
    int i;

    ret = 0;
    for (i=0; i<2; i++) {
	ret *= 0x10;
	c = str[i];
	if (isdigit(c))
	    ret += c - '0';
	else if (c >= 'a' && c <= 'f')
	    ret += c - 'a' + 10;
	else if (c >= 'A' && c <= 'F')
	    ret += c - 'A' + 10;
	else
	    return -1;
    }
    return ret;
}

static char *
int2str(int num)
{
    char tmp[20];
    static char ret[21];
    register char *p1 = tmp, *p2 = ret;
    
    if (num < 0) {
	*p2++ = '-';
	num = -num;
    }
    do {
	*p1++ = '0'+(num % 10);
	num /= 10;
    } while (num != 0);
    
    while (p1 != tmp)
	*p2++ = *--p1;
    *p2 = '\0';
    
    return ret;
}

static void
load_error(int line, char *message)
{
    s_puts(CRLF "S-Record loading failed (line:");
    s_puts(int2str(line));
    s_puts(") : ");
    s_puts(message);
    s_puts(CRLF CRLF);
    s_puts("Skip garbage lines until you press [ESC] key..." CRLF);
    while (1) {
	int c = s_getc();
	if (c == ESC)
	    break;
	if (c == '\n')
	    continue;
	s_putc(c);
    }
}

int
monitor_load()
{
    char buffer[LOAD_BUFFER_SIZE];
    int bytes[LOAD_BUFFER_SIZE];
    int name, start, count, index, line;
    long bytecount;
    register int i;
    register unsigned char FAR * volatile ptr;
    register unsigned char sum;
    enum { small, medium, large } address;
    line = 0;
    bytecount = 0;
    
    s_puts("Please send Motorola S-Record file" CRLF);
    s_puts("Press [ESC] and return key for stop loading" CRLF CRLF);
    s_puts("loading ...  ");
    
    for (i=0; i<FILENAME_LEN; i++)
	filename[i] = '\0';
    while (TRUE) {
	s_gets(buffer, sizeof(buffer));
	
	if (buffer[0] == '\r' || buffer[0] == '\n')
	    continue;
	if (buffer[0] == ESC) {
	    s_puts(CRLF CRLF "Interrupted!!" CRLF);
	    return FALSE;
	}
	line++;
	if (buffer[0] != 'S') {
	    load_error(line, "record type");
	    return FALSE;
	}
	name = start = FALSE;
	switch (buffer[1]) {
	case '0': name = TRUE;				break;
	case '1': address = small;			break;
	case '2': address = medium;			break;
	case '3': address = large;			break;
	case '7': address = large;  start = TRUE;	break;
	case '8': address = medium; start = TRUE;	break;
	case '9': address = small;  start = TRUE;	break;
	default:
	    load_error(line, "record type");
	    return FALSE;
	}
	count = hex2toint(&buffer[2]);
	if (count >= LOAD_BUFFER_SIZE) {
	    load_error(line, "record bytecount is wrong");
	    return FALSE;
	}
	sum = count;
	for (i=0; i<count; i++) {
	    bytes[i] = hex2toint(&buffer[4 + i*2]);
	    sum += bytes[i];
	}
	if (sum != 0xFF) {
	    load_error(line, "check sum");
	    return FALSE;
	}
	count--; /* checksum byte is not used in below lines */
	if (name) {
	    sum = 0;
	    for (i=0,index=0; i<FILENAME_LEN-1 && index<count; index++) {
		if (bytes[index] != '\0') {
		    filename[i++] = bytes[index];
		    sum += bytes[index];
		}
	    }
	    filename[FILENAME_LEN-1] = ~sum;
	    continue;
	}
	switch (address) {
	case small:
	    ptr = (unsigned char *)
		((((unsigned long)bytes[0]) << 8)|
		 ((unsigned long)bytes[1]));
	    index = 2;
	    break;
	case medium:
	    ptr = (unsigned char *)
		((((unsigned long)bytes[0]) << 16)|
		 (((unsigned long)bytes[1]) << 8) |
		 ((unsigned long)bytes[2]));
	    index = 3;
	    break;
	case large:
	    ptr = (unsigned char *)
		((((unsigned long)bytes[0]) << 24)|
		 (((unsigned long)bytes[1]) << 16)|
		 (((unsigned long)bytes[2]) << 8) |
		 ((unsigned long)bytes[3]));
	    index = 4;
	    break;
	}
	if (count < index) {
	    load_error(line, "record too short");
	    return FALSE;
	}
	if (start) {
	    start_addr = ptr;
	    break;
	}
	for (i=index; i<count; i++)
	    *ptr++ = bytes[i];
	
	if ((line % 10) == 0) {
	    s_putc(BACKSPACE);
	    s_putc(TWIRL[(line/10)%TWIRL_NUM]);
	}
	bytecount += count - index;
    }
    s_puts("ok" CRLF);
    
    s_puts("S-Record file : \"");
    for (i=0; i<FILENAME_LEN-1 && filename[i]!='\0'; i++)
	s_putc(filename[i]);
    s_puts("\"  ");
    s_puts(int2str(line));
    s_puts(" lines, ");
    s_puts(int2str(bytecount));
    s_puts(" bytes loaded" CRLF);
    return TRUE;
}

