/* $Id: mon_main.c,v 1.5 2002/01/16 16:26:02 amura Exp $ */
/*
 *  Copyright (c) 2001,2002    MURAMATSU Atsushi
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * A Simple Monitor Program
 *   - main and utility routines
 */

#include "macdep.h"
#include "monitor_in.h"
#include "monitor.h"

#include <stdlib.h>
#include <ctype.h>

#ifndef BOARD_NAME
#if defined(AP_SH3)
#define BOARD_NAME	"AlphaProject AP-SH3"
#elif defined(AKI_H8)
#define BOARD_NAME	"Akizuki AKI-H8"
#else
#define BOARD_NAME	"Unknown board"
#endif
#endif /* !BOARD_NAME */
#define COPYRIGHT	"A Simple Monitor Program ver.0.1 for " BOARD_NAME \
	"\r\n        Copyright(c) 2001,2002  MURAMATSU Atsushi\r\n\r\n"

void FAR *go_addr;
static char prompt[FILENAME_LEN];

void
monitor_run(char *arg)
{
    go_addr = start_addr;
    if (arg != NULL)
	go_addr = (void FAR *)strtol(arg, NULL, 16);
    MONITOR_LAMP(OFF);
    start();
    MONITOR_LAMP(ON);
}

void
monitor_help()
{
    s_puts(COPYRIGHT);
    s_puts(CRLF "HELP:" CRLF);
    s_puts("    load         ... download program from host" CRLF);
    s_puts("    run [addr]   ... run program" CRLF);
    s_puts("    dump [addr]  ... memory binary dump" CRLF);
    s_puts("    edit [addr]  ... memory edit" CRLF);
#ifdef HAVE_SUB_SRAM
    s_puts("    store [addr] ... save program to SRAM" CRLF);
    s_puts("    restore      ... load program from SRAM" CRLF);
#endif
    s_puts("    help         ... this help" CRLF CRLF);
}

#ifdef HAVE_SUB_SRAM
static unsigned short
calc_checksum(register unsigned char FAR * volatile ptr, unsigned long size)
{
    unsigned long i;
    register short sum;
    sum = 0;
    for (i=0; i<size; i++)
	sum = (sum + *ptr++) & 0xFFFF;
    return sum;
}

void
monitor_store(char *arg)
{
    unsigned char FAR * volatile addr = DEFAULT_SAVE_ADDRESS;
    if (arg != NULL)
	addr = (unsigned char FAR *)strtol(arg, NULL, 16);
    s_puts("Calculate checksum... ");
    *SRAM_ADDRESS_PTR  = addr;
    *SRAM_CHECKSUM_PTR = calc_checksum(addr, SRAM_SIZE);
    s_puts(CRLF "Storing data to SRAM... ");
    memcpy(SRAM_ADDR, addr, SRAM_SIZE);
    s_puts("DONE." CRLF);
}

int
monitor_restore(void)
{
    unsigned short checksum;
    checksum = calc_checksum(SRAM_ADDR, SRAM_SIZE);
    if (checksum != *SRAM_CHECKSUM_PTR) {
	s_puts("SRAM Check sum is not valid." CRLF);
	s_puts("RAM Restore is not succeed." CRLF);
	return FALSE;
    }
    s_puts("SRAM Check sum is valid .. now RAM Restore" CRLF);
    memcpy(*SRAM_ADDRESS_PTR, SRAM_ADDR, SRAM_SIZE);
    s_puts("RAM Restore is succeed." CRLF);
    return TRUE;
}
#endif /* HAVE_SUB_SRAM */

static void
set_prompt()
{
    register int i;
    register unsigned char sum;
    
    sum = 0;
    for (i=0; i<FILENAME_LEN; i++) {
	prompt[i] = filename[i];
	sum += filename[i];
    }
    if (sum == 0xFF)
	prompt[FILENAME_LEN-1] = '\0';
    else
	prompt[0] = '\0';
}

static int
cmd_check(char *buffer, const char *str)
{
    while (*buffer) {
	if (*buffer != *str)
	    return FALSE;
	buffer++; str++;
    }
    return TRUE;
}

static int
str_split(char *args[], int max_args_num, register char *buf)
{
    register int i;
    int n = 0;
    while (*buf) {
	/* Skip scape charactors */
	if (isspace(*buf)) {
	    buf++;
	    continue;
	}
	args[n] = buf;
	while (!isspace(*buf))
	    buf++;
	*buf++ = '\0'; /* terminate string */
	if (++n >= max_args_num)
	    break;
    }
    for (i=n; i<max_args_num; i++)
	args[i] = NULL;
    return n;
}

int
main()
{
    char buffer[LINE_BUFFER_SIZE];
    char *args[ARGS_MAX];
    int len;

    monitor_init();
    s_puts(COPYRIGHT);
#ifdef HAVE_SUB_SRAM
    monitor_restore();
#endif
    set_prompt();
    
    MONITOR_LAMP(ON);
    while (TRUE) {
	s_puts(prompt);
	s_puts(PROMPT);
	
	len = s_gets_echo_with_go(buffer, sizeof(buffer));
	s_puts(CRLF);
	if (len == -1)
	    strcpy(buffer, "go\r\n");
	str_split(args, ARGS_MAX, buffer);
	
	if (args[0] == NULL)
	    continue;
	if (cmd_check(args[0], "load")) {
	    monitor_load();
	    set_prompt();
	}
	else if (cmd_check(args[0], "run"))
	    monitor_run(args[1]);
	else if (cmd_check(args[0], "dump"))
	    monitor_dump(args[1], args[2]);
	else if (cmd_check(args[0], "edit"))
	    monitor_edit(args[1]);
#ifdef HAVE_SUB_SRAM
	else if (cmd_check(args[0], "store"))
	    monitor_store(args[1]);
	else if (cmd_check(args[0], "restore")) {
	    monitor_restore();
	    set_prompt();
	}
#endif
	else {
	    if (!cmd_check(args[0], "help"))
		s_puts("WARNING: unknown command" CRLF);
	    monitor_help();
	}
    }
}
