/* $Id: mon_mem.c,v 1.2 2002/01/13 02:44:31 amura Exp $ */
/*
 *  Copyright (c) 2001,2002    MURAMATSU Atsushi
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * A Simple Monitor Program
 *   - memory dump and edit routines
 */

#include <stdlib.h>

#include "macdep.h"
#include "monitor_in.h"
#include "monitor.h"

#define MAX_HEXSTR_LEN	20
static char tmp_buffer[MAX_HEXSTR_LEN];

static char *
int2hexstr(unsigned long data, int len)
{
    register int i;
    for (i=len-1; i>=0; i--) {
	tmp_buffer[i] = "0123456789ABCDEF"[data & 0x0F];
	data >>= 4;
    }
    tmp_buffer[len] = '\0';
    return tmp_buffer;
}

void
monitor_dump(char *addr_str, char *len_str)
{
    static unsigned char FAR *before_addr;
    int len = 128;
    register int i, lower;
    unsigned char FAR * volatile addr = before_addr;
    if (addr_str != NULL)
	addr = (unsigned char FAR *)strtol(addr_str, NULL, 16);
    if (len_str != NULL) {
	len = strtol(len_str, NULL, 10);
	if (len <= 0)
	    len = 128;
    }
    
    /* output memory dump header */
    s_puts(" address :");
    lower = ((unsigned long)addr) & 0x0F;
    for (i=0; i<16; i++) {
	if (i == 8)	s_putc('-');
	else		s_putc(' ');
	s_puts(int2hexstr(lower++, 2));
    }
    s_puts(CRLF);
    for (i=0; i<(8 + 2 + 3*16); i++)
	s_putc('-');

    /* memory dump main */
    for (i=0; i<len; i++) {
	switch (i & 0x0F) {
	case 0x00:
	    s_puts(CRLF);
	    s_puts(int2hexstr((unsigned long)addr, 8));
	    s_puts(" : ");
	    break;
	case 0x08:
	    s_putc('-');
	    break;
	default:
	    s_putc(' ');
	}
	s_puts(int2hexstr(*addr++, 2));
    }
    s_puts(CRLF);
    before_addr = addr;
}

void
monitor_edit(char *addr_str)
{
    static unsigned char FAR *before_addr;
    unsigned char FAR * volatile addr = before_addr;
    if (addr_str != NULL)
	addr = (unsigned char *)strtol(addr_str, NULL, 16);
    while (TRUE) {
	s_puts(int2hexstr((unsigned long)addr, 8));
	s_puts(" : ");
	s_puts(int2hexstr(*addr, 2));
	s_puts(" -> ");
	s_gets_echo(tmp_buffer, sizeof(tmp_buffer));
	if (tmp_buffer[0] == ESC) 
	    break;
	*addr++ = (strtol(tmp_buffer, NULL, 16) & 0xFF);
	s_puts(CRLF);
    }
    s_puts(CRLF);
    before_addr = addr;
}
