/* $Id: vector_sh3.h,v 1.2 2002/01/13 02:46:49 amura Exp $ */
/*
 *  Copyright (c) 2001,2002    MURAMATSU Atsushi
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
/*
 * A Simple Monitor Program Header
 *   - machine indepedent macros around interrupt vector
 */

#ifndef __VECTOR_H__
#define __VECTOR_H__

#define DEFAULT_VBR_ADDRESS	((void *)0x80000000)

#define VIRTUAL_VECTOR_ADDR	(0x8CFE0000)

#define INTR_VECTOR	((volatile void **)(VIRTUAL_VECTOR_ADDR + 0x000))
#define TRAP_VECTOR	((volatile void **)(VIRTUAL_VECTOR_ADDR + 0x800))

#define	INTR_VECTOR_MAX		255
#define	TRAP_VECTOR_MAX		256

/* Interrrupt Vector table */
#define INTR_NMI_VECTOR		(0x1C0u >> 5)
#define INTR_HUDI_VECTOR	(0x5E0u >> 5)

#define INTR_IRQ0_VECTOR	(0x600u >> 5)
#define INTR_IRQ1_VECTOR	(0x620u >> 5)
#define INTR_IRQ2_VECTOR	(0x640u >> 5)
#define INTR_IRQ3_VECTOR	(0x660u >> 5)
#define INTR_IRQ4_VECTOR	(0x680u >> 5)
#define INTR_IRQ5_VECTOR	(0x6A0u >> 5)

#define INTR_PINT0_VECTOR	(0x700u >> 5)
#define INTR_PINT8_VECTOR	(0x720u >> 5)

#define INTR_DMAC0_VECTOR	(0x800u >> 5)
#define INTR_DMAC1_VECTOR	(0x820u >> 5)
#define INTR_DMAC2_VECTOR	(0x840u >> 5)
#define INTR_DMAC3_VECTOR	(0x860u >> 5)

#define INTR_SCI1_ERI_VECTOR	(0x880u >> 5)
#define INTR_SCI1_RXI_VECTOR	(0x8A0u >> 5)
#define INTR_SCI1_BRI_VECTOR	(0x8C0u >> 5)
#define INTR_SCI1_TXI_VECTOR	(0x8E0u >> 5)
#define INTR_SCI2_ERI_VECTOR	(0x900u >> 5)
#define INTR_SCI2_RXI_VECTOR	(0x920u >> 5)
#define INTR_SCI2_BRI_VECTOR	(0x940u >> 5)
#define INTR_SCI2_TXI_VECTOR	(0x960u >> 5)

#define INTR_ADC_VECTOR		(0x980u >> 5)

#define INTR_TMU0_VECTOR	(0x400u >> 5)
#define INTR_TMU1_VECTOR	(0x420u >> 5)
#define INTR_TMU2_VECTOR	(0x440u >> 5)
#define INTR_TMU2_ICPI_VECTOR	(0x460u >> 5)

#define INTR_RTC_ATI_VECTOR	(0x480u >> 5)
#define INTR_RTC_PRI_VECTOR	(0x4A0u >> 5)
#define INTR_RTC_CUI_VECTOR	(0x4C0u >> 5)

#define INTR_SCI0_ERI_VECTOR	(0x4E0u >> 5)
#define INTR_SCI0_RXI_VECTOR	(0x500u >> 5)
#define INTR_SCI0_TXI_VECTOR	(0x520u >> 5)
#define INTR_SCI0_TEI_VECTOR	(0x540u >> 5)

#define INTR_WDT_VECTOR		(0x560u >> 5)
#define INTR_BSC_RCMI_VECTOR	(0x580u >> 5)
#define INTR_BSC_ROVI_VECTOR	(0x5A0u >> 5)

#define set_trap_handler(n, ptr)	(TRAP_VECTOR[(n)] = (void *)(ptr))
#define get_trap_handler(n)		(TRAP_VECTOR[(n)])
#define set_ill_handler(ptr)		(set_trap_handler(TRAP_VECTOR_MAX, ptr))
#define get_ill_handler()		(get_trap_handler(TRAP_VECTOR_MAX))
#define set_intr_handler(n, ptr)	(INTR_VECTOR[(n)] = (void *)(ptr))
#define get_intr_handler(n)		(INTR_VECTOR[(n)])

#define MONITOR_LAMP(flag)		(PJ.DR.BIT.P7DT = (flag) ? 1 : 0)

#ifdef __GNUC__	/* Assembler routines for GNU C Compiler */

#define DISABLE()	({					\
    __asm volatile("stc sr,r0\n\tor %0,r0\n\tldc r0,sr":	\
	/*NO Output*/: "r"(0x10000000) : "r0");			\
})
#define ENABLE()	({					\
    __asm volatile("stc sr,r0\n\tand %0,r0\n\tldc r0,sr":	\
	/*NO Output*/: "r"(0xEFFFFFFF) : "r0");			\
})
#define set_IMASK(mask)	({						\
    __asm volatile("stc sr,r0\n\tand %0,r0\n\tor %1,r0\n\tldc r0,sr":	\
	/*NO Output*/: "r"(0xEFFFFF0F), "r"(((mask)&0x0F)<<4) : "r0");	\
})
#define get_IMASK()	({					\
    register int _sr;						\
    __asm volatile("stc sr,%0": "=r"(_sr)); ((_sr>>4)&0x0F);	\
})
#define set_VBR(ptr)	({						\
    __asm volatile ("ldc %0,vbr": /*NO Output*/ : "r"((int)ptr));	\
})
#define get_VBR()	({				\
    register int _r;					\
    __asm__("stc vbr,%0": "=r"(_r)); (void *)_r;	\
})

#else /* Assember routines for Hitachi SH-C Compiler */

#pragma inline_asm(DISABLE)
static void DISABLE(void) {
    mov    #$10000000,r1
    stc    sr,r0
    or     r1,r0
    ldc    r0,sr
}
#pragma inline_asm(ENABLE)
static void ENABLE(void) {
    mov    #$EFFFFFFF,r1
    stc    sr,r0
    and    r1,r0
    ldc    r0,sr
}
#pragma inline_asm(set_IMASK)
static void set_IMASK(int mask) {
    mov    #$0F,r1
    and    r1,r4
    shll4  r4
    mov    #$EFFFFF0F,r1
    stc    sr,r0
    and    r1,r0
    or     r4,r0
    ldc    r0,sr
}
#pragma inline_asm(get_IMASK)
static int get_VBR(void) {
    stc    sr,r0
    shlr4  r0
    and    #$0F,r0
}
#pragma inline_asm(set_VBR)
static void set_VBR(void *ptr) {
    mov.l  r4,r0
    ldc    r0,vbr
}
#pragma inline_asm(get_VBR)
static void *get_VBR(void) {
    stc    vbr,r0
}

#endif /* __GNUC__ */

#endif /* __VECTOR_H__ */
